@@ -62,11 +62,45 @@ GtkWidget	*w_default_load;
 
 
 
+static gboolean
+defui_question_message (const gchar *msg)
+{
+    GtkWidget 	*dialog = NULL;
+    gboolean 	rv = FALSE;
+    gchar	*message = NULL;
+    gint 	response;
+    
+    message = g_strdup_printf (_("Are you sure to load default values for %s."), msg);
+    
+    dialog = gtk_message_dialog_new (
+	    NULL,
+	    GTK_DIALOG_DESTROY_WITH_PARENT,
+	    GTK_MESSAGE_INFO,GTK_BUTTONS_YES_NO,
+	    message);
+
+    response = gtk_dialog_run (GTK_DIALOG (dialog));
+    
+    if (response == GTK_RESPONSE_YES)
+	rv = TRUE;
+    
+    gtk_widget_destroy (dialog);
+    
+    g_free (message);
+    
+    return rv;
+}
+
 
 static void 
 defui_default_braille_clicked	(GtkButton       *button,
                                  gpointer         user_data)
 {
+    if (*(gboolean*)user_data)
+    {
+	if (!defui_question_message (_("braille")))
+	    return;
+    }
+    
     brlconf_load_default_settings (braille_setting);
     brlconf_setting_set (braille_setting);
     
@@ -86,6 +120,12 @@ static void
 defui_default_speech_clicked (GtkButton       *button,
                               gpointer         user_data)
 {
+    if (*(gboolean*)user_data)
+    {
+	if (!defui_question_message (_("speech")))
+	    return;
+    }
+    
     spconf_load_default_settings (speech_setting);
     spconf_setting_set (speech_setting);
     spui_speech_setting_value_add_to_widgets (speech_setting);
@@ -97,6 +137,11 @@ static void
 defui_default_magnifier_clicked	(GtkButton       *button,
                                  gpointer         user_data)
 {
+    if (*(gboolean*)user_data)
+    {
+	if (!defui_question_message (_("magnifier")))
+	    return;
+    }
 
     if (magnifier_setting) 
     {
@@ -115,6 +160,12 @@ static void
 defui_default_keyboard_clicked (GtkButton       *button,
                                 gpointer         user_data)
 {
+    if (*(gboolean*)user_data)
+    {
+	if (!defui_question_message (_("keyboard")))
+	    return;
+    }
+
     kbconf_load_default_settings (keyboard_setting);
     kbconf_setting_set ();
     kbui_keyboard_settings_value_add_to_widgets (keyboard_setting);
@@ -126,6 +177,12 @@ static void
 defui_default_braille_monitor_clicked (GtkButton       *button,
                             	       gpointer         user_data)
 {
+    if (*(gboolean*)user_data)
+    {
+	if (!defui_question_message (_("braille monitor")))
+	    return;
+    }
+
     bmconf_load_default_settings ();
     bmui_braille_monitor_values_add_to_widgets ();
     if (*(gboolean*)user_data)
@@ -136,12 +193,18 @@ static void
 defui_default_general_clicked (GtkButton       *button,
                                gpointer         user_data)
 {
+    if (*(gboolean*)user_data)
+    {
+	if (!defui_question_message (_("start up")))
+	    return;
+    }
+
     srcore_load_default_settings (general_setting);
     srcore_general_setting_set   (general_setting);
     genui_value_add_to_widgets 	 (general_setting);
     
     if (*(gboolean*)user_data)
-	gn_show_message (_("General default setting loaded."));
+	gn_show_message (_("Startup default setting loaded."));
 }
 
 
@@ -149,6 +212,12 @@ static void
 defui_default_cmdmap_clicked (GtkButton       *button,
                              gpointer         user_data)
 {
+    if (*(gboolean*)user_data)
+    {
+	if (!defui_question_message (_("command map")))
+	    return;
+    }
+    
     cmdconf_remove_lists_from_gconf ();
     cmdconf_default_list (TRUE);
     cmdconf_changes_end_event ();
@@ -161,6 +230,12 @@ defui_default_pres_clicked (GtkButton       *button,
                             gpointer         user_data)
 
 {
+    if (*(gboolean*)user_data)
+    {
+	if (!defui_question_message (_("presentation")))
+	    return;
+    }
+
     presconf_set_defaults ();
     if (*(gboolean*)user_data)
 	gn_show_message (_("Presentation default setting loaded."));
@@ -170,6 +245,12 @@ static void
 defui_default_scr_rev_clicked (GtkButton       *button,
                                gpointer         user_data)
 {
+    if (*(gboolean*)user_data)
+    {
+	if (!defui_question_message (_("screen review")))
+	    return;
+    }
+
     srcore_load_default_screen_review ();
     if (*(gboolean*)user_data)
 	gn_show_message (_("Screen review default setting loaded."));
@@ -179,6 +260,7 @@ void
 defui_load_all_default (void)
 {
     gboolean bval = FALSE;
+    
     defui_default_braille_clicked (NULL, (gpointer)&bval);
     defui_default_braille_monitor_clicked (NULL, (gpointer)&bval);
     defui_default_speech_clicked (NULL, (gpointer)&bval);
@@ -194,6 +276,8 @@ static void
 defui_default_all_clicked (GtkButton       *button,
                            gpointer         user_data)
 {
+    if (!defui_question_message (_("all modules")))
+	return;
     defui_load_all_default ();
     gn_show_message (_("All default settings loaded."));
 }
