@@ -119,7 +119,6 @@ static gint 	gn_semid;
  *
 **/
 static gint	gn_srcore_pid;
-extern gint 	bm_pid;
 
 /**
  *
@@ -319,7 +318,7 @@ main (gint argc,gchar *argv[])
         sru_warning (_("GTK initialization failed."));
         return EXIT_FAILURE;
     }
-	
+    
     if (!gn_run_srcore ())
     {
         sru_warning (_("Can not create a new process."));
@@ -456,6 +455,10 @@ gn_run_with_parameters (void)
     
     if (gn_braille_device)
     {
+	gchar *tmp = NULL;
+	tmp = g_utf8_strup (gn_braille_device, -1);
+	g_free (gn_braille_device);
+	gn_braille_device = tmp;
     	gn_argv[index] = g_strdup_printf ("--braille-device=%s", gn_braille_device);
 	brlconf_device_set (gn_braille_device);
 	index++;
@@ -602,19 +605,11 @@ gn_init (void)
     speech_setting    = spconf_setting_init (TRUE);
     general_setting   = srcore_general_setting_init (TRUE);
 
-    if (general_setting->braille_monitor)
-    {
-	genui_run_brlmonitor ();
-	sleep (1);
-    }
-	
     srcore_exit_all (FALSE);
     
     if (gn_loaddefault) 
 	defui_load_all_default ();
-/*    
-    lng_set_envirom (srcore_language_get ());
-*/
+
     return TRUE;
 }
 
@@ -678,6 +673,7 @@ gn_run_srcore (void)
 	    sru_message ("No \"%s\" binary file found [errno: %i].",
 			"srcore",
 			errno);
+	    g_strfreev (gn_argv);
 	    return FALSE;
 	}
     }
@@ -695,6 +691,7 @@ gn_run_srcore (void)
 	    sru_message ("No \"%s\" binary file found [errno: %i].",
 			"srcore",
 			errno);
+	    g_strfreev (gn_argv);
 	    return FALSE;
 	}
     }
@@ -708,6 +705,7 @@ gn_run_srcore (void)
 	    sru_message ("No \"%s\" binary file found [errno: %i].",
 			"srcore",
 			errno);
+	    g_strfreev (gn_argv);
 	    return FALSE;
 	}
     }
@@ -813,13 +811,8 @@ gn_child_process_wait (void)
 #endif
 	gn_srcore_pid = 0;
     }
+
     
-    /* Emit terminate signal to BrlMonitor that it to be sure it is exited */
-    if (bm_pid != 0)
-    {
-	kill (bm_pid, SIGTERM);
-    }
-	
     /* Emit terminate signal to SRCore that it to be sure it is exited */
     if (gn_srcore_pid != 0)
     {
