@@ -26,7 +26,19 @@
 #include "srintl.h"
 #include "gnopiui.h"
 
+#define MAGUI_ALIGNMENT_NONE		"none"
+#define MAGUI_ALIGNMENT_CENTERED	"centered"
+#define MAGUI_ALIGNMENT_AUTO		"auto"
+#define MAGUI_ALIGNMENT_MIN		"min"
+#define MAGUI_ALIGNMENT_MAX		"max"
 
+enum {
+    MAG_FOCUS_TRACKING_NONE = 0,
+    MAG_FOCUS_TRACKING_CENTERED,
+    MAG_FOCUS_TRACKING_AUTO,
+    
+    MAG_FOCUS_TRACKING_NUMBER
+};
 struct
 {
     gchar *description;
@@ -37,11 +49,13 @@ struct
     {N_("bilinear"), 	"bilinear"}
 };
 
-struct
+typedef struct
 {
     gchar *description;
     gchar *key;
-} magui_mouse_tracking [] = 
+} TrackingNameStruct;
+
+TrackingNameStruct magui_mouse_tracking [] = 
 {
     {N_("push"), 		"mouse-push"},
     {N_("centered"),		"mouse-centered"},
@@ -108,40 +122,55 @@ GtkWidget    	*sp_zp_left,
 GtkWidget	*cp_border_color,
 		*cp_crosswire_color,
 		*cp_cursor_color;
+GtkWidget	*rb_focus_tracking[MAG_FOCUS_TRACKING_NUMBER];
 
-void 
+static void 
 magui_cursor_off (gboolean val)
 {
+    gboolean cursor_size = val;
+    gboolean crosswire_clip = val;
+	
     gtk_widget_set_sensitive ( GTK_WIDGET (ck_cursor_mag), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (cp_cursor_color), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (sp_cursor_size), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_cursor_color), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (sp_cursor_size), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (lb_cursor_size), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (ck_crosswire), val);    
     
     if (val)
-    {
-	val = magnifier_setting->crosswire;
-    }
-    gtk_widget_set_sensitive ( GTK_WIDGET (ck_crosswire_clip), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (sp_crosswire_size), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (cp_crosswire_color), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (lb_crosswire_size), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (lb_crosswire_color), val);
+	cursor_size = !(magnifier_setting->cursor_scale);
+		
+    gtk_widget_set_sensitive (GTK_WIDGET (sp_cursor_size), cursor_size);
+    gtk_widget_set_sensitive (GTK_WIDGET (lb_cursor_size), cursor_size);
+	
+    if (val)
+    	crosswire_clip = magnifier_setting->crosswire;
+	
+    gtk_widget_set_sensitive ( GTK_WIDGET (ck_crosswire_clip), crosswire_clip);
 }
 
-void 
+static void
+magui_cursor_magnification_off (gboolean val) 
+{
+    gtk_widget_set_sensitive ( GTK_WIDGET (sp_cursor_size), val);
+    gtk_widget_set_sensitive ( GTK_WIDGET (lb_cursor_size), val);
+}
+
+static void 
 magui_crosswire_off (gboolean val)
 {
+	
     gtk_widget_set_sensitive ( GTK_WIDGET (ck_crosswire_clip), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (sp_crosswire_size), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (cp_crosswire_color), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_crosswire_size), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_crosswire_color), val);
+    gtk_widget_set_sensitive ( GTK_WIDGET (ck_crosswire_color_invert), val);    
+    
+    if (val)
+	val = magnifier_setting->cursor_state;
+	
+    gtk_widget_set_sensitive ( GTK_WIDGET (ck_crosswire_clip), val);
 }
 
-
+#if 0
 void 
 magui_save_magnifier_setting_value (Magnifier *magnifier_setting)
 {    
@@ -170,7 +199,7 @@ magui_save_magnifier_setting_value (Magnifier *magnifier_setting)
     magnifier_setting->cursor_state = 
 	    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ck_cursor_on_off));
 
-    magnifier_setting->cursor_mag = 
+    magnifier_setting->cursor_scale = 
 	    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ck_cursor_mag));
 	
     magnifier_setting->cursor_size =	    
@@ -187,15 +216,15 @@ magui_save_magnifier_setting_value (Magnifier *magnifier_setting)
 
     magconf_save_zoomer_in_schema (DEFAULT_MAGNIFIER_SCHEMA, magnifier_setting);
 }
+#endif
 
 void
 magui_magnification_settings_response (GtkDialog *dialog,
 				       gint       response_id,
 				       gpointer   user_data)
 {
-    if (response_id == GTK_RESPONSE_OK) 
+    if (response_id == GTK_RESPONSE_CLOSE) 
     {
-	magui_save_magnifier_setting_value (magnifier_setting);
     }
     else
     if (response_id == GTK_RESPONSE_HELP) 
@@ -264,13 +293,14 @@ magui_cursor_magnification_toggled (GtkWidget *widget,
 {
     gboolean bval = 
 	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ck_cursor_mag)); 	    
-    if (magnifier_setting->cursor_mag != bval)
+    if (magnifier_setting->cursor_scale != bval)
     {
-	magnifier_setting->cursor_mag = bval;
+	magnifier_setting->cursor_scale = bval;
 	magconf_save_item_in_schema (DEFAULT_MAGNIFIER_SCHEMA,
 				    GCONF_VALUE_BOOL,
-				    MAGNIFIER_CURSOR_MAG,
-				    (gpointer)&magnifier_setting->cursor_mag);
+				    MAGNIFIER_CURSOR_SCALE,
+				    (gpointer)&magnifier_setting->cursor_scale);
+	magui_cursor_magnification_off (!magnifier_setting->cursor_scale);
     }
 }
 
@@ -775,7 +805,7 @@ magui_set_cursor_mag (Magnifier *magnifier_setting)
 	return FALSE;
 
     gtk_toggle_button_set_active (  GTK_TOGGLE_BUTTON (ck_cursor_mag), 
-				    magnifier_setting->cursor_mag);
+				    magnifier_setting->cursor_scale);
 
     return TRUE;
 }
@@ -839,8 +869,10 @@ magui_magnifier_setting_value_add_to_widgets (Magnifier *magnifier_setting)
     return TRUE;
 }
 
-
-
+/**
+		Cursor option UI.
+**/
+#if 0
 void
 magui_magnification_options_save_changed (void)
 {
@@ -949,20 +981,20 @@ magui_magnification_options_save_changed (void)
 	
     magconf_save_zoomer_in_schema (DEFAULT_MAGNIFIER_SCHEMA, magnifier_setting);
 }
-
+#endif
 void
 magui_magnification_options_response (GtkDialog *dialog,
 				      gint       response_id,
 				      gpointer   user_data)
 {
-    if (response_id == GTK_RESPONSE_OK) 
+    if (response_id == GTK_RESPONSE_CLOSE) 
     {
         if (mag_setting_clone_opt)
             {
                 magconf_setting_free (mag_setting_clone_opt);
 		mag_setting_clone_opt = NULL;
 	    }
-	magui_magnification_options_save_changed ();
+	/*magui_magnification_options_save_changed ();*/
     }
     else
     if (response_id == GTK_RESPONSE_HELP) 
@@ -1376,6 +1408,39 @@ magui_change_lock_factor(GtkWidget       *widget,
 				(gpointer)&magnifier_setting->zoomfactorx);				
 }
 
+static void
+magui_rb_focus_tracking_toggled (GtkWidget	*widget,
+				gpointer 	user_data)
+{
+    g_free (magnifier_setting->alignment_x);
+    g_free (magnifier_setting->alignment_y);
+    
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rb_focus_tracking[MAG_FOCUS_TRACKING_NONE])))
+    {
+    	magnifier_setting->alignment_x = g_strdup (MAGUI_ALIGNMENT_NONE);
+	magnifier_setting->alignment_y = g_strdup (MAGUI_ALIGNMENT_NONE);
+    }
+    else
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rb_focus_tracking[MAG_FOCUS_TRACKING_CENTERED])))
+    {
+	magnifier_setting->alignment_x = g_strdup (MAGUI_ALIGNMENT_CENTERED);
+	magnifier_setting->alignment_y = g_strdup (MAGUI_ALIGNMENT_CENTERED);
+    }
+    else
+    {
+    	magnifier_setting->alignment_x = g_strdup (MAGUI_ALIGNMENT_AUTO);
+	magnifier_setting->alignment_y = g_strdup (MAGUI_ALIGNMENT_AUTO);
+    }
+    
+    magconf_save_item_in_schema (DEFAULT_MAGNIFIER_SCHEMA,
+				 GCONF_VALUE_STRING,
+				 MAGNIFIER_ALIGNMENT_X,
+				(gpointer)magnifier_setting->alignment_x);				
+    magconf_save_item_in_schema (DEFAULT_MAGNIFIER_SCHEMA,
+				 GCONF_VALUE_STRING,
+				 MAGNIFIER_ALIGNMENT_Y,
+				(gpointer)magnifier_setting->alignment_x);				
+}
 
 			
 void
@@ -1401,6 +1466,10 @@ magui_set_handlers_magnification_options(GladeXML *xml)
 
     et_source  = glade_xml_get_widget (xml, "et_source");
     et_target  = glade_xml_get_widget (xml, "et_target");
+	
+    rb_focus_tracking[MAG_FOCUS_TRACKING_NONE] = glade_xml_get_widget (xml, "rb_focus_tracking_none");
+    rb_focus_tracking[MAG_FOCUS_TRACKING_CENTERED] = glade_xml_get_widget (xml, "rb_focus_tracking_centered");
+    rb_focus_tracking[MAG_FOCUS_TRACKING_AUTO] = glade_xml_get_widget (xml, "rb_focus_tracking_auto");
 
     gtk_spin_button_set_range ( GTK_SPIN_BUTTON (sp_factorx), MIN_ZOOM_FACTOR, MAX_ZOOM_FACTOR);
     gtk_spin_button_set_range ( GTK_SPIN_BUTTON (sp_factory), MIN_ZOOM_FACTOR, MAX_ZOOM_FACTOR);
@@ -1461,6 +1530,13 @@ magui_set_handlers_magnification_options(GladeXML *xml)
 			    GTK_SIGNAL_FUNC (on_change_sp2_value));
     glade_xml_signal_connect (xml, "on_ck_lock_factor_toggled",			
 			    GTK_SIGNAL_FUNC (magui_change_lock_factor));
+				
+    glade_xml_signal_connect (xml, "on_rb_focus_tracking_none_clicked",
+				GTK_SIGNAL_FUNC (magui_rb_focus_tracking_toggled));
+    glade_xml_signal_connect (xml, "on_rb_focus_tracking_centered_clicked",
+				GTK_SIGNAL_FUNC (magui_rb_focus_tracking_toggled));
+    glade_xml_signal_connect (xml, "on_rb_focus_tracking_auto_clicked",
+				GTK_SIGNAL_FUNC (magui_rb_focus_tracking_toggled));
 }
 
 
@@ -1724,6 +1800,34 @@ magui_set_mouse_tracking (Magnifier *magnifier_setting)
 
 
 
+
+gboolean
+magui_set_focus_tracking (Magnifier *magnifier_setting)
+{
+    sru_return_val_if_fail (magnifier_setting, FALSE);
+    
+    if (!w_magnification_options) 
+	return FALSE;
+	
+    if (!strcmp (magnifier_setting->alignment_x, MAGUI_ALIGNMENT_NONE) &&
+	!strcmp (magnifier_setting->alignment_x, MAGUI_ALIGNMENT_NONE))	
+    {
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_focus_tracking[MAG_FOCUS_TRACKING_NONE]), TRUE);
+    }
+    else
+    if (!strcmp (magnifier_setting->alignment_x, MAGUI_ALIGNMENT_CENTERED) &&
+	!strcmp (magnifier_setting->alignment_x, MAGUI_ALIGNMENT_CENTERED))	
+    {
+    	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_focus_tracking[MAG_FOCUS_TRACKING_CENTERED]), TRUE);
+    }
+    else
+    {
+    	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rb_focus_tracking[MAG_FOCUS_TRACKING_AUTO]), TRUE);
+    }			
+    
+    return TRUE;
+}
+
 gboolean
 magui_set_invert (Magnifier *magnifier_setting)
 {
@@ -1750,8 +1854,6 @@ magui_set_panning (Magnifier *magnifier_setting)
     return TRUE;
 }
 
-
-
 gboolean
 magui_magnification_options_value_add_to_widgets (Magnifier *magnifier_setting)
 {
@@ -1784,8 +1886,8 @@ magui_magnification_options_value_add_to_widgets (Magnifier *magnifier_setting)
     magui_set_smoothing (magnifier_setting);
 			    
     magui_set_mouse_tracking (magnifier_setting);
+	
+	magui_set_focus_tracking (magnifier_setting);
 
     return TRUE;
 }
-
-
