@@ -65,6 +65,7 @@ static SRConfStatusEnum srconf_status = SRCONF_IDLE;
  *
 **/
 GConfClient *gconf_client=NULL;
+GConfEngine *gconf_engine=NULL;
 
 /**
  * gconf_root_dir_path:
@@ -74,6 +75,7 @@ GConfClient *gconf_client=NULL;
 **/
 static gchar *gconf_root_dir_path = NULL;
 
+static gboolean use_config_settings = FALSE;
 /**
  * DEFAULT_ROOT_DIR_PATH:
  *
@@ -234,8 +236,7 @@ sr_config_structure_destructor (gpointer data)
  *
 **/
 static void 
-sr_config_changed_callback (GConfClient *client, 
-			    guint cnxn_id, 
+sr_config_changed_callback (guint cnxn_id, 
 			    GConfEntry *entry, 
 			    gpointer user_data)
 {
@@ -360,6 +361,24 @@ sr_config_changed_callback (GConfClient *client,
     sre_release_reference (evnt);
     sru_debug ("SendEventSrCore");
 }
+
+static void 
+sr_config_client_changed_callback (GConfClient *client, 
+			    guint cnxn_id, 
+			    GConfEntry *entry, 
+			    gpointer user_data)
+{
+    sr_config_changed_callback (cnxn_id, entry, user_data);
+}
+static void 
+sr_config_engine_changed_callback (GConfEngine *engine, 
+			    guint cnxn_id, 
+			    GConfEntry *entry, 
+			    gpointer user_data)
+{
+    sr_config_changed_callback (cnxn_id, entry, user_data);
+}
+
 /**
  * srconf_init:
  *
@@ -368,7 +387,9 @@ sr_config_changed_callback (GConfClient *client,
  *	gconf_root_dir_path - the root directory of GConf
 **/
 gboolean 
-srconf_init (SRConfCB srconfcb, const gchar *_gconf_root_dir_path)
+srconf_init (SRConfCB srconfcb, 
+	     const gchar *_gconf_root_dir_path,
+	     const gchar *config_source)
 {
     gint i;
     GError *error = NULL;    
@@ -376,52 +397,93 @@ srconf_init (SRConfCB srconfcb, const gchar *_gconf_root_dir_path)
     
     sru_return_val_if_fail (srconf_status == SRCONF_IDLE, FALSE);
     sru_return_val_if_fail (srconfcb != NULL, FALSE);
-
+    
     srconf_fnc = srconfcb;
 
     gconf_root_dir_path = g_strdup (_gconf_root_dir_path?_gconf_root_dir_path: DEFAULT_ROOT_DIR_PATH);
     
     sru_return_val_if_fail (gconf_root_dir_path != NULL, FALSE);
 
-    /*set a default gconf client		      */
-    gconf_client = gconf_client_get_default ();
-    
-    /*adding the directory to be watched		      */
-    gconf_client_add_dir (gconf_client, gconf_root_dir_path, GCONF_CLIENT_PRELOAD_ONELEVEL, &error);
-    
-    
-    if (error != NULL)
-    {
-	sru_warning(_("Failed to add directory."));
-	sru_warning(_(error->message));
-	g_error_free(error);
-	error = NULL;
-    }
-	
-    /*adding notification callbacks*/
-    for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
-    {
-    	path = g_strdup_printf ("%s%s", gconf_root_dir_path, srconf_notify_directories[i].notify_path);
-    	srconf_notify_directories[i].cnx_id = gconf_client_notify_add (gconf_client, path, sr_config_changed_callback, (gpointer)(i), NULL, &error);
+    /*set a default gconf client */
+    if (!config_source)
+    {	
+	use_config_settings = FALSE;
+        gconf_client = gconf_client_get_default ();
 	
+	/*adding the directory to be watched*/
+	gconf_client_add_dir (gconf_client, gconf_root_dir_path, GCONF_CLIENT_PRELOAD_ONELEVEL, &error);
 	if (error != NULL)
 	{
-	    sru_warning (_("Failed to add notify."));
-	    sru_warning (_(error->message));
-	    g_error_free (error);
+	    sru_warning(_("Failed to add directory."));
+	    sru_warning(_(error->message));
+	    g_error_free(error);
 	    error = NULL;
 	}
+	
+	for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+	{
+    	    path = g_strdup_printf ("%s%s", gconf_root_dir_path, srconf_notify_directories[i].notify_path);
+    	    srconf_notify_directories[i].cnx_id = gconf_client_notify_add (gconf_client, path, sr_config_client_changed_callback, (gpointer)(i), NULL, &error);
+	
+	    if (error != NULL)
+	    {
+		sru_warning (_("Failed to add notify."));
+		sru_warning (_(error->message));
+		g_error_free (error);
+		error = NULL;
+	    }
 
-    	g_free (path);
-    	path = NULL;
+    	    g_free (path);
+    	    path = NULL;
+	}
+
+    }
+    else
+    {
+	path = g_strdup_printf ("xml:readwrite:%s", config_source);
+    	gconf_engine = gconf_engine_get_for_address (path, &error);
+	g_free (path);
+	
+	use_config_settings = TRUE;
+/*      conf = gconf_engine_get_local (config_source, &error);*/
+	
+	if (gconf_engine == NULL)
+	{
+    	    sru_assert (error != NULL);
+    	    sru_warning(_("Failed to access configuration source(s): %s\n"), error->message);
+    	    g_error_free (error);
+    	    error = NULL;
+    	    return FALSE;
+	}
+	
+	for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+	{
+    	    path = g_strdup_printf ("%s%s", gconf_root_dir_path, srconf_notify_directories[i].notify_path);
+    	    srconf_notify_directories[i].cnx_id = gconf_engine_notify_add (gconf_engine, path, sr_config_engine_changed_callback, (gpointer)(i),  &error);
+	
+	    if (error != NULL)
+	    {
+		sru_warning(_("Failed to add notify.:%s"), error->message);
+		sru_warning(_(error->message));
+		g_error_free (error);
+		error = NULL;
+	    }	
+
+    	    g_free (path);
+    	    path = NULL;
+	}
+	
+	gconf_client = gconf_client_get_for_engine (gconf_engine);
+	
+	gconf_engine_unref (gconf_engine);
     }
         
+    
     srconf_status = SRCONF_RUNNING;
     sru_debug ("SRConf running.");
             
     return TRUE;
 }
-
 /**
  * srconf_terminate:
  *
@@ -435,12 +497,23 @@ srconf_terminate()
     GError *error = NULL;
     sru_return_if_fail (srconf_status == SRCONF_RUNNING);
     
-    /* removing notification callbacks */
-    for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
-    	gconf_client_notify_remove (gconf_client, srconf_notify_directories[i].cnx_id);
+    if (!use_config_settings)
+    {
+	/* removing notification callbacks */
+	for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+    	    gconf_client_notify_remove (gconf_client, srconf_notify_directories[i].cnx_id);
     	
-    /* remove the watching directory */
-    gconf_client_remove_dir (gconf_client, gconf_root_dir_path, &error);
+	/* remove the watching directory */
+	gconf_client_remove_dir (gconf_client, gconf_root_dir_path, &error);
+    }
+    else
+    {
+	/* removing notification callbacks */
+	for( i=0; i < NUM_OF_CONFIGURABLES; i++ )
+    	    gconf_engine_notify_remove (gconf_engine, srconf_notify_directories[i].cnx_id);
+	    
+	gconf_engine_unref (gconf_engine);
+    }
     
     if (error != NULL)
     {
@@ -551,6 +624,8 @@ srconf_get_config_data_with_default(const gchar *key, SRConfigTypesEnum conftype
 		*((gpointer*)data) = NULL;
 	        used_default = FALSE;
 	    }
+	    
+	    gconf_value_free (value);
 	}
 	else
 	{
@@ -679,6 +754,8 @@ srconf_get_data_with_default(const gchar *key, SRConfigTypesEnum conftype, gpoin
 		*((gpointer*)data) = NULL;
 	        used_default = FALSE;
 	    }
+	    
+	    gconf_value_free (value);
 	}
 	else
 	{
@@ -907,3 +984,22 @@ srconf_unset_key (const gchar *key,const gchar *section)
     
     return retval;
 }
+
+gchar*
+srconf_presentationi_get_chunk (const gchar *device_role_event)
+{
+    gchar *active = NULL;
+    gchar *chunk  = NULL;
+    
+    sru_assert (device_role_event);
+
+    srconf_get_data_with_default(PRESENTATION_ACTIVE_SETTING, CFGT_STRING, &active ,DEFAULT_PRESENTATION_ACTIVE_SETTING_NAME , PRESENTATION_PATH);
+    if (active)
+    {
+	gchar *key = g_strconcat (active, "/", device_role_event, NULL);    
+	srconf_get_data_with_default (key, CFGT_STRING, &chunk ,DEFAULT_PRESENTATION_ACTIVE_SETTING_NAME , PRESENTATION_PATH);   
+	g_free (key);	
+    }
+
+    return chunk;
+}
