@@ -37,6 +37,7 @@
 #include "srpres.h"
 #include "srintl.h"
 #include "screen-review.h"
+#include <libxml/parser.h>
 
 
 static SRLClientHandle src_srl_client;
@@ -61,6 +62,7 @@ extern gboolean panning;
 
 extern gint src_braille_offset;
 
+
 void
 src_present_null_sro ()
 {
@@ -237,7 +239,12 @@ src_present_real_sro (SRObject *obj)
 		txt = src_presentation_present_sro_for_device (obj, reason, "speech");
     		if (txt && txt[0])
     		{
-		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE);
+		    SRObjectRoles role;
+		    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+		    if (role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
+			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, FALSE);
+		    else
+			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 		}
 		g_free (txt);
 	    }
@@ -251,6 +258,11 @@ src_present_real_sro (SRObject *obj)
 	txt = src_presentation_present_sro_for_device (obj, reason, "magnifier");
 	if (txt)
 	{
+/*	    fprintf (stderr, "\n(%s %d) txt = %s",
+			__FILE__,
+			__LINE__,
+			txt);
+*/
 	    gchar *tmp;
 	    tmp = g_strconcat ("<MAGOUT><ZOOMER ID=\"generic_zoomer\" tracking=\"focus\" ",
 				txt,
@@ -301,7 +313,10 @@ src_present_crt_window ()
 	txt = src_presentation_present_sro_for_device (src_crt_window, reason, "speech");
     	if (txt && txt[0])
     	{
-	    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM);
+	    if (strcmp (reason, "window:create") == 0)
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_WARNING, FALSE);
+	    else
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
 	}
 	g_free (txt);
     }
@@ -327,7 +342,7 @@ src_present_crt_tooltip ()
 	txt = src_presentation_present_sro_for_device (src_crt_tooltip, reason, "speech");
     	if (txt && txt[0])
     	{
-	    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM);
+	    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
 	}
 	g_free (txt);
     }
@@ -394,7 +409,7 @@ src_present_details ()
     }    
     
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     
     g_free (name);
     g_free (role);
@@ -434,7 +449,7 @@ src_present_layer_changed ()
 		
 	message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), tmp);
 	if (message && message[0])
-	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 
 	g_free (tmp);
 	g_free (message);
@@ -455,7 +470,7 @@ src_present_layer_timeout ()
 	
 	message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), tmp);
 	if (message && message[0])
-	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 
 	g_free (tmp);
 	g_free (message);
@@ -985,7 +1000,7 @@ static gboolean src_crosswire	    	= FALSE;
     }
     else
     if (config->key && 
-	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_CURSOR_MAG) == 0)
+	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_CURSOR_SCALE) == 0)
     {
 	if (src_use_magnifier && 
 	    config->newvalue &&
@@ -1606,7 +1621,56 @@ static gboolean src_crosswire	    	= FALSE;
 		src_magnifier_stop_panning (0);
 	}
     }
+    else
+    if (config->key && 
+	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_ALIGNMENT_X) == 0)
+    {
+    	if (src_use_magnifier && 
+	    config->newvalue &&
+	    config->type == CFGT_STRING);
+	{
+	    gchar *alignment_x = (gchar*)config->newvalue;
+	
+	    if (src_use_magnifier) 
+	    {
+		gchar *mag_out = NULL;
+		
+		mag_out = g_strdup_printf("<MAGOUT><ZOOMER ID=\"generic_zoomer\"  AlignmentX=\"%s\"></ZOOMER></MAGOUT>", alignment_x);
 
+		if (mag_out)
+		{
+		    src_magnifier_send (mag_out);
+		    g_free (mag_out);
+		    mag_out = NULL;
+		}
+	    }
+	}
+    }
+    else
+    if (config->key && 
+	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_ALIGNMENT_Y) == 0)
+    {
+    	if (src_use_magnifier && 
+	    config->newvalue &&
+	    config->type == CFGT_STRING);
+	{
+	    gchar *alignment_y = (gchar*)config->newvalue;
+	
+	    if (src_use_magnifier) 
+	    {
+		gchar *mag_out = NULL;
+		
+		mag_out = g_strdup_printf ("<MAGOUT><ZOOMER ID=\"generic_zoomer\"  AlignmentY=\"%s\"></ZOOMER></MAGOUT>", alignment_y);
+
+		if (mag_out)
+		{
+		    src_magnifier_send (mag_out);
+		    g_free (mag_out);
+		    mag_out = NULL;
+		}		
+	    }
+	}
+    }
 }
 
 static void
@@ -1621,11 +1685,101 @@ src_process_config_changed_for_control (SRConfigStructure *config)
     /*src_update_key ();*/
 }
 
+static xmlNode *
+load_presentation_file (gchar *fname) 
+{
+    xmlDoc *doc;
+    xmlNode *rootnode;
+    if ((doc = xmlParseFile (fname)) && (rootnode = xmlDocGetRootElement (doc))) 
+    {
+	if (xmlSearchNsByHref (doc, rootnode, (const xmlChar *) "http://www.gnome.org/gnopernicus") != NULL)
+	{
+	    if (xmlStrcmp (rootnode->name, (const xmlChar *) "PresentationProfile") == 0) 
+	    {
+		return rootnode;
+	    }
+	    else
+	    {
+		fprintf (stderr, "Root element is not a PresentationProfile\n.");
+	    }
+	}
+	else
+	{
+	    fprintf (stderr, "Error: incorrect namespace in presentation file.\n");	    
+	}
+    }
+    else 
+    {
+	fprintf (stderr, doc ? "file %s not found\n" : "file %s is empty.\n", fname ? fname : "<nil>" );
+	xmlFreeDoc (doc);
+	return NULL;
+    }
+    return NULL;
+}
+
+static void
+set_presentation_gconf_key (gchar *key, gchar *value)
+{
+    srconf_set_data (key, CFGT_STRING, value, PRESENTATION_PATH);
+}
+
+static gint
+set_gconf_keys_from_xml (xmlNode *root)
+{
+    gboolean found = FALSE;
+    xmlNode *node;
+    const GList *lang_list = gnome_i18n_get_language_list ("LC_MESSAGES");
+
+    while (!found && lang_list) 
+    {
+	for (node = root->xmlChildrenNode; node != NULL; node = node->next)
+	{
+	    if (xmlStrcmp (node->name, (const xmlChar *) "_presentation-key") == 0) 
+	    {
+		char *lang = xmlNodeGetLang (node);
+		char *locale = lang ? lang : "C";
+		if (!xmlStrcmp (lang_list->data, locale))
+		{
+		    gchar *key;
+		    gchar *value;
+		    found = TRUE;
+		    if (lang) xmlFree (lang);
+		    key = xmlGetProp (node, (const xmlChar *) "type");
+		    value = xmlNodeGetContent (node);
+		    if (key && value) 
+		    {
+			set_presentation_gconf_key (key, value);
+			xmlFree (value);
+		    }
+		}
+	    }
+	}
+	lang_list = lang_list->next;
+    }
+
+    return !found;
+}
+
+
 static void
 src_process_config_changed_for_presentation (SRConfigStructure *config)
 {
     if (config)
     {
+	xmlNode *root;
+	gchar *name = "default";
+	gchar *name2;
+	
+	srconf_get_data_with_default (PRESENTATION_ACTIVE_SETTING, CFGT_STRING, &name, &name, PRESENTATION_PATH);
+
+	name2 = g_strconcat (PRESENTATION_DIR, name, ".xml", NULL); 
+	root = load_presentation_file (name2);
+	g_free (name);
+	g_free (name2);
+
+	if (root)
+	    set_gconf_keys_from_xml (root);
+
 	src_presentation_terminate ();
 	src_presentation_init ();
     }
@@ -1753,7 +1907,7 @@ src_kb_key_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_key_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_MESSAGE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_MESSAGE, TRUE);
     g_free (message); 
     
     return TRUE;    
@@ -1769,7 +1923,7 @@ src_kb_punct_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_punct_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message); 
     
     return TRUE;    
@@ -1785,7 +1939,7 @@ src_kb_space_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_space_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message); 
     
     return TRUE;    
@@ -1801,7 +1955,7 @@ src_kb_modifier_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_modifier_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message); 
     
     return TRUE;    
@@ -1818,7 +1972,7 @@ src_kb_cursor_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_cursor_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_SYSTEM);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
     g_free (message); 
 
     return TRUE;    
@@ -2422,6 +2576,7 @@ src_get_defaults ()
     
     if ((gint)src_use_braille == -1)
     {
+	src_use_braille = DEFAULT_SRCORE_BRAILLE_ACTIVE;
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_ACTIVE, CFGT_BOOL, 
 	    &src_use_braille, &src_use_braille);
     }
@@ -2429,18 +2584,21 @@ src_get_defaults ()
     
     if ((gint)src_use_braille_monitor == -1)
     {
+	src_use_braille_monitor = DEFAULT_SRCORE_BRAILLE_MONITOR_ACTIVE;
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_MONITOR_ACTIVE, CFGT_BOOL, 
 	    &src_use_braille_monitor, &src_use_braille_monitor);
     }
     
     if ((gint)src_use_speech == -1)
     {
+	src_use_speech = DEFAULT_SRCORE_SPEECH_ACTIVE;
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, 
 					    &src_use_speech, &src_use_speech);
     }
 
     if ((gint)src_use_magnifier == -1)
     {
+	src_use_magnifier = DEFAULT_SRCORE_MAGNIF_ACTIVE;
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_MAGNIF_ACTIVE, CFGT_BOOL, 
 					    &src_use_magnifier, &src_use_magnifier);
     }
@@ -2479,9 +2637,95 @@ src_present_last_message ()
     return TRUE;
 }
 
+void
+src_set_command_lines (const gchar *mouse_tracking,
+		       const gchar *focus_tracking,
+		       gboolean    invert,
+		       gint 	   zoomfactor)
+{
+    if (mouse_tracking)
+    {
+    	gchar *mt = g_ascii_strdown (mouse_tracking, -1);
+	if (!strcmp (mt, "centered"))
+	{
+	    g_free (mt);
+	    mt = g_strdup ("mouse-centered");
+	}
+	else if (!strcmp (mt, "proportional"))
+	{
+	    g_free (mt);
+	    mt = g_strdup ("mouse-proportional");
+	}
+	else if (!strcmp (mt, "push"))
+	{
+	    g_free (mt);
+	    mt = g_strdup ("mouse-push");
+	}
+	else if (!strcmp (mt, "none"))
+	{
+	    g_free (mt);
+	    mt = g_strdup ("none");
+	}
+	else
+	{
+	    g_free (mt);
+	    mt = NULL;
+	}
+	if (mt)
+	{
+	    srconf_set_data (MAGNIFIER_MOUSE_TRACKING,
+			  CFGT_STRING, 
+			  mt,
+			  MAGNIFIER_CONFIG_PATH);
+	
+	}
+	g_free (mt);
+    }
+    
+    if (focus_tracking)
+    {
+	gchar *ft = g_ascii_strdown (focus_tracking, -1);
+	if (!strcmp (ft, "none") ||
+	    !strcmp (ft, "centered") ||
+	    !strcmp (ft, "auto"))
+	{
+	    srconf_set_data (MAGNIFIER_ALIGNMENT_Y,
+			  CFGT_STRING, 
+			  ft,
+			  MAGNIFIER_CONFIG_PATH);
+    
+	    srconf_set_data (MAGNIFIER_ALIGNMENT_X,
+			  CFGT_STRING, 
+			  ft,
+			  MAGNIFIER_CONFIG_PATH);
+	}
+	g_free (ft);
+    }
+    
+    srconf_set_data (MAGNIFIER_INVERT,
+			  CFGT_BOOL, 
+			  &invert,
+			  MAGNIFIER_CONFIG_PATH);
+    
+    if (0 < zoomfactor && 17 > zoomfactor)
+    {
+    	srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
+			  CFGT_INT, 
+			  &zoomfactor,
+			  MAGNIFIER_CONFIG_PATH);
+    	srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
+			  CFGT_INT, 
+			  &zoomfactor,
+			  MAGNIFIER_CONFIG_PATH);
+    }
+}    
 
 static void
-src_init()
+src_init(const gchar *mouse_tracking,
+	 const gchar *focus_tracking,
+	 const gchar *config_source,
+	 gboolean    invert,
+	 gint 	     zoomfactor)
 {
     SRLClient client;
     gchar *message = _("Welcome to Gnopernicus");
@@ -2492,14 +2736,14 @@ src_init()
     sru_init();
 
     /* initialize SRConf */
-    srconf_init((SRConfCB)src_event_sink, "/apps/gnopernicus");
+    srconf_init((SRConfCB)src_event_sink, "/apps/gnopernicus", config_source);
 
     /* layout init */
     src_ctrl_init ();
     
 /*    src_speech_count_mode = DEFAULT_SPEECH_REPEAT_TYPE;*/
     src_get_defaults ();
-
+    
     /* initialize Braille */
     if (src_use_braille)
     {
@@ -2514,6 +2758,12 @@ src_init()
     if (src_use_speech)
 	src_use_speech = src_speech_init ();
 
+    /* set the command lines arguments in gconf */
+    src_set_command_lines (mouse_tracking,
+		    	    focus_tracking,
+			    invert,
+			    zoomfactor);
+
     /* initialize Magnifier */
     if (src_use_magnifier)
     {
@@ -2615,8 +2865,7 @@ src_terminate()
     /* terminate Speech */
     if (src_use_speech)
 	src_speech_terminate ();
-
-
+	
     /* terminate SRConf */
     SET_SRCORE_CONFIG_DATA(SRCORE_EXIT_ACK_KEY, CFGT_BOOL, &exitack);
     exitack = !exitack;
@@ -2642,7 +2891,7 @@ src_terminate()
     if (src_last_cursor_echo)
 	g_free (src_last_cursor_echo);
 }
-
+    
 gint
 main (gint argc, 
       gchar **argv)
@@ -2655,6 +2904,14 @@ main (gint argc,
     gboolean disable_magnifier	= FALSE;
     gboolean enable_braille_monitor 	= FALSE;
     gboolean disable_braille_monitor 	= FALSE;    
+    
+    gchar 	*config_source 	= NULL;
+    gchar 	*mouse_tracking = NULL;
+    gchar	*focus_tracking = NULL;
+    gint  	zoom_factor     = 0;
+    gboolean 	invert 	    	= FALSE;
+
+    
 
     struct poptOption poptopt[] = 
     {		
@@ -2664,7 +2921,7 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &enable_braille,		
 	 0, 
-	 "enable braille service", 
+	 "Enable braille service", 
 	 NULL
 	},
 	{
@@ -2673,7 +2930,7 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &disable_braille,		
 	 0, 
-	 "enable braille service", 
+	 "Disable braille service", 
 	 NULL
 	},
 	{
@@ -2682,7 +2939,7 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &enable_speech,		
 	 0, 
-	 "enable speech service", 
+	 "Enable speech service", 
 	 NULL
 	},
 	{
@@ -2691,7 +2948,7 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &disable_speech,		
 	 0, 
-	 "disable speech service", 
+	 "Disable speech service", 
 	 NULL
 	},
 	{
@@ -2700,7 +2957,7 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &enable_magnifier,		
 	 0, 
-	 "enable magnifier service",  
+	 "Enable magnifier service",  
 	 NULL
 	},
 	{
@@ -2709,7 +2966,7 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &disable_magnifier,		
 	 0, 
-	 "disable magnifier service",  
+	 "Disable magnifier service",  
 	 NULL
 	},
 
@@ -2719,7 +2976,7 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &enable_braille_monitor, 	
 	 0, 
-	 "enable braille monitor service", 
+	 "Enable braille monitor service", 
 	 NULL
 	},
 	{
@@ -2728,7 +2985,7 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &disable_braille_monitor, 	
 	 0, 
-	 "disable braille monitor service", 
+	 "Disable braille monitor service", 
 	 NULL
 	},
 	{
@@ -2749,6 +3006,53 @@ main (gint argc,
 	 "Braille Device", 
 	 "DEVICE NAME"
 	},
+	{
+	"settings-dir", 
+	 'c', 
+	 POPT_ARG_STRING, 
+	 &config_source, 	
+	 0, 
+	 "Use gconf settings file", 
+	 "PATH"
+	},
+	{
+	"magnification-zoom", 
+	 'z', 
+	 POPT_ARG_INT, 
+	 &zoom_factor, 	
+	 0, 
+	 "Set magnification zoom level", 
+	 "[1..16]"
+	},
+	{
+	"magnification-invert", 
+	 'i', 
+	 POPT_ARG_NONE, 
+	 &invert, 	
+	 0, 
+	 "Magnification with reverse video", 
+	 ""
+	},
+
+	{
+	"magnification-mouse-tracking", 
+	 't', 
+	 POPT_ARG_STRING, 
+	 &mouse_tracking, 	
+	 0, 
+	 "How the mouse pushed the region", 
+	 "[CENTERED, PROPORTION, PUSH, NONE]"
+	},
+
+	{
+	"magnification-focus-tracking", 
+	 'f', 
+	 POPT_ARG_STRING, 
+	 &focus_tracking, 	
+	 0, 
+	 "How the focus pushed the region", 
+	 "[AUTO, CENTERED, NONE]"
+	},
 	{
 	 NULL, 0,     0, NULL, 0
 	}
@@ -2793,7 +3097,21 @@ main (gint argc,
     src_use_magnifier = (disable_magnifier ? FALSE :
 			(enable_magnifier  ? TRUE : -1));
 
-    src_init ();
+    if (src_braille_device)
+    {
+	gchar *tmp = NULL;
+	tmp = g_utf8_strup (src_braille_device, -1);
+	g_free (src_braille_device);
+	src_braille_device = tmp;
+    }
+        
+
+    src_init (mouse_tracking, focus_tracking, config_source, invert, zoom_factor);
+    
+    g_free (mouse_tracking);
+    g_free (focus_tracking);
+    g_free (config_source);
+
 
     sru_entry_loop ();
 
