@@ -510,7 +510,7 @@ src_speech_say_ (gchar *text_,
     if  (attr)
 	text = src_xml_format ("TEXT", attr, text_);
     if (text)
-	rv = src_speech_send_chunk (text, priority);
+	rv = src_speech_send_chunk (text, priority, TRUE);
     g_free (text);
     g_free (attr);
 
@@ -1037,20 +1037,17 @@ src_speech_resume ()
 
 
 gboolean
-src_speech_send_chunk (gchar *chunk, SRCSpeechPriority priority)
+src_speech_send_chunk (gchar *chunk, SRCSpeechPriority priority, gboolean preempt)
 {
     gchar *output;
     gboolean rv = FALSE;
-    static SRCSpeechPriority last_priority = SRC_SPEECH_PRIORITY_IDLE;
 
     sru_assert (src_speech_initialized);
     sru_assert (chunk && chunk[0]);
-    if (last_priority >= priority)
-	src_speech_shutup ();
-    output = g_strconcat ("<SRSOUT>", chunk, "</SRSOUT>", NULL);
+    output = g_strdup_printf ("<SRSOUT priority=\"%d\" preempt=\"%s\">%s</SRSOUT>",
+			priority, preempt ? "true" : "false", chunk);
     rv = src_speech_send (output);
     g_free (output);
-    last_priority = priority;
 
     return rv;
 }
@@ -1438,7 +1435,17 @@ src_speech_translate_symbol (gint index)
     sru_assert (index >= 0 && index < G_N_ELEMENTS (src_speech_dictionary));
 
     if (src_speech_dictionary[index].translate)
-	rv = g_strdup (src_speech_dictionary[index].singular);
+    {
+	if (src_speech_dictionary[index].character == '.' ||
+	    src_speech_dictionary[index].character == '!' ||
+	    src_speech_dictionary[index].character == '?')
+	{
+	    rv = g_strconcat ("  ", src_speech_dictionary[index].singular, " ", NULL);
+	    rv[0] = src_speech_dictionary[index].character;
+	}
+	else
+	    rv = g_strconcat (" ", src_speech_dictionary[index].singular, " ", NULL);
+    }
     else
     {
 	rv = g_strdup (" ");
@@ -1503,8 +1510,14 @@ src_speech_process_string (const gchar   *text,
 	
     if (text[0] == ' ' && text[1] == '\0' && smode == SRC_SPEECH_SPELL_NORMAL)
 	return g_strdup (_("space"));
-    if (text[0] && text[1] == '\0')
-	return g_strdup (text);
+    if (text[0] && !text[1])
+    {
+        gint index = src_speech_get_index_for_symbol (crt[0]);
+	if (index != -1)
+	    return g_strdup (src_speech_dictionary[index].singular);
+	else
+	    return g_strdup ((gchar*)text);
+    }
 
     if (src_speech_dictionary_type == SRC_SPEECH_DICTIONARY_YES)
     {
