@@ -345,6 +345,7 @@ typedef enum _SRObjectRole
 	SR_ROLE_TABLE_LINE,
 	SR_ROLE_TABLE_COLUMNS_HEADER,
 	SR_ROLE_TITLE_BAR,
+	SR_ROLE_EDITBAR,
     
     /* not a valid role, used for finding end of enumeration. */
     SR_ROLE_LAST_DEFINED
@@ -523,6 +524,7 @@ gboolean sro_get_next_image (SRObject *obj, SRObject **next, SRNavigationMode mo
 gboolean sro_get_next_attributes (SRObject *obj, gchar *attr, SRObject **next, SRNavigationMode mode);
 
 gboolean sro_get_sros_from_rectangle (SRRectangle *area, GArray **array, gboolean intersect);
+gboolean sro_alert_get_info (SRObject *obj, gchar **title, gchar **text, gchar **button);
 
 #ifdef SRL_DEBUG
 void SR_freeString 	(gchar *str);
